@@ -31,13 +31,13 @@ from urllib.parse import quote as url_quote
 from uuid import UUID
 
 import httpx
-from pydantic import BaseModel, EmailStr, Field, JsonValue
+from pydantic import AliasPath, BaseModel, EmailStr, Field, JsonValue
 
 if TYPE_CHECKING:
     from typing import Self  # For Python ≤ 3.10
 
 
-__version__ = "0.6"
+__version__ = "0.7"
 
 DEFAULT_BASE_URL = "https://api.smartextract.ai"
 DEFAULT_TIMEOUT = 600  # seconds
@@ -215,10 +215,16 @@ class TemplateInfo(BaseInfo):
 
     id: str = Field(
         description="Template identifier, in the document_type.language form.",
-        examples=["invoice.en", "bank_statement.de"],
+        examples=["invoice", "bank_statement"],
+    )
+    name: str = Field(
+        description="Localized name of the template.",
+        validation_alias=AliasPath("display_name", "en"),
+    )
+    description: str = Field(
+        description="Localized description of the template.",
+        validation_alias=AliasPath("description", "en"),
     )
-    name: str = Field(description="Localized name of the template.")
-    description: str = Field(description="Localized description of the template.")
     categories: list = Field(
         description="List of business domains relevant to this template."
     )
@@ -408,7 +414,7 @@ class AsyncClient:
     async def list_templates(self, language: Language = "en") -> list[TemplateInfo]:
         """List all available templates in format name.language."""
         r = await self._request("GET", "/templates", params={"lang": language})
-        return [TemplateInfo(**template) for template in r.json()]
+        return [TemplateInfo.model_validate(v) for v in r.json()]
 
     async def get_user_info(self, user: str = "me") -> UserInfo:
         """Request stored information and credit balance of a given user."""
@@ -599,6 +605,7 @@ class AsyncClient:
         name: str,
         code: str,
         *,
+        template: str | None = None,
         permissions: Optional[dict[str, AccessLevel]] = None,
     ) -> UUID:
         """Create a new pipeline by providing a lua script.
@@ -611,7 +618,12 @@ class AsyncClient:
         r = await self._request(
             "POST",
             "/pipelines",
-            json=drop_none(name=name, code=code, permissions=permissions),
+            json=drop_none(
+                name=name,
+                code=code,
+                template=template,
+                permissions=permissions,
+            ),
         )
         return UUID(r.json()["id"])
 
@@ -979,7 +991,7 @@ class Client:
     def list_templates(self, language: Language = "en") -> list[TemplateInfo]:
         """List all available templates in format name.language."""
         r = self._request("GET", "/templates", params={"lang": language})
-        return [TemplateInfo(**template) for template in r.json()]
+        return [TemplateInfo.model_validate(v) for v in r.json()]
 
     def get_user_info(self, user: str = "me") -> UserInfo:
         """Request stored information and credit balance of a given user."""
@@ -1166,6 +1178,7 @@ class Client:
         name: str,
         code: str,
         *,
+        template: str | None = None,
         permissions: Optional[dict[str, AccessLevel]] = None,
     ) -> UUID:
         """Create a new pipeline by providing a lua script.
@@ -1178,7 +1191,12 @@ class Client:
         r = self._request(
             "POST",
             "/pipelines",
-            json=drop_none(name=name, code=code, permissions=permissions),
+            json=drop_none(
+                name=name,
+                code=code,
+                template=template,
+                permissions=permissions,
+            ),
         )
         return UUID(r.json()["id"])
 
