@@ -11,6 +11,7 @@ import os
 import pathlib
 import sys
 from collections.abc import Callable
+from enum import Enum
 from typing import Any
 
 from pydantic import JsonValue, TypeAdapter
@@ -112,33 +113,23 @@ def get_dumper(args: argparse.Namespace) -> Callable:
     return dump
 
 
-def json_argument(data_or_file: str) -> JsonValue:
+def json_data(data_or_file: str) -> JsonValue:
     """A CLI argument type accepting a file name and returning its content as JSON."""
-    if data_or_file.startswith("@"):
-        file = argparse.FileType("r")(data_or_file[1:])
-    else:
-        file = None
+    try:
+        return json.loads(data_or_file)
+    except json.JSONDecodeError:
+        file = argparse.FileType("r")(data_or_file)
     try:
         return json.load(file) if file else json.loads(data_or_file)
     except Exception as e:
-        where = file.name if file else "command line"
         raise argparse.ArgumentTypeError(
-            f"Error reading JSON data from {where}: {e}"
+            f"Error reading JSON data from {file.name}: {e}"
         ) from e
 
 
-json_argument_help = "either a literal JSON value or @FILENAME to read from a file"
+json_argument_help = "either a literal JSON value or a file to read"
 
-
-def template_argument(template: str) -> JsonValue:
-    """CLI type for extraction templates (template ID of JSON file name)."""
-    return template[1:] if template.startswith("#") else json_argument(template)
-
-
-template_argument_help = (
-    "extraction template (literal JSON value or #NAME.LANG to use"
-    " a predefined template or @FILENAME to read from a file)"
-)
+template_argument_help = f"extraction template, {json_argument_help}"
 
 
 def key_value_argument(s: str) -> tuple[str, str]:
@@ -151,9 +142,26 @@ def key_value_argument(s: str) -> tuple[str, str]:
 
 ## CLI definition
 
+
+class HelpFormatter(argparse.RawDescriptionHelpFormatter):
+    """Help formatter with nice metavar names."""
+
+    def _get_default_metavar_for_optional(self, action):
+        if action.help and action.help.startswith("ID or alias"):
+            return "ID"
+        if isinstance(action.type, type) and issubclass(action.type, Enum):
+            return "{" + ",".join(str(v.value) for v in action.type) + "}"
+        if isinstance(action.type, argparse.FileType):
+            return "FILE"
+        type_name = getattr(action.type, "__name__", None)
+        if type_name:
+            return type_name.upper()
+        return action.dest.upper()
+
+
 cli = argparse.ArgumentParser(
     description="Make requests to the smartextract API.",
-    formatter_class=argparse.RawDescriptionHelpFormatter,
+    formatter_class=HelpFormatter,
 )
 cli.add_argument(
     "-v",
@@ -226,7 +234,7 @@ def subcommand(
     subcmd = subcommands.add_parser(
         name,
         aliases=aliases,
-        formatter_class=argparse.RawDescriptionHelpFormatter,
+        formatter_class=HelpFormatter,
         **kwargs,
     )
     subcmd.set_defaults(handler=handler)
@@ -275,11 +283,12 @@ set_user_credits = subcommand(
     handler=do_set_user_credits,
 )
 set_user_credits.add_argument("username", help="email or ID of the user")
-set_user_credits.add_argument("--balance", "-b", help="set a new balance")
+set_user_credits.add_argument("--balance", "-b", help="set a new balance", type=int)
 set_user_credits.add_argument(
     "-c",
     "--new-credits",
     help="add credits to current balance",
+    type=int,
 )
 set_user_credits.add_argument(
     "-s",
@@ -315,7 +324,7 @@ get_resource_info = subcommand(
         get_client(args).get_resource_info(args.id_or_alias)
     ),
 )
-get_resource_info.add_argument("id_or_alias", help="Resource UUID or resource alias")
+get_resource_info.add_argument("id_or_alias", help="resource ID or alias")
 
 list_resources = subcommand(
     "list-resources",
@@ -338,7 +347,7 @@ list_resources.add_argument(
         "openai_chat",
         "inbox",
     ],
-    help="Filter by resource type.",
+    help="filter by resource type",
 )
 
 list_lua_pipelines = subcommand(
@@ -373,7 +382,7 @@ list_permissions = subcommand(
         get_client(args).list_permissions(args.id_or_alias)
     ),
 )
-list_permissions.add_argument("id_or_alias", help="Resource UUID or resource alias")
+list_permissions.add_argument("id_or_alias", help="resource ID or alias")
 
 
 def _do_create_permission(args):
@@ -425,6 +434,11 @@ create_lua_pipeline = subcommand(
 create_lua_pipeline.add_argument(
     "-n", "--name", default="Lua pipeline", help="name of the new pipeline"
 )
+create_lua_pipeline.add_argument(
+    "--template",
+    type=json_data,
+    help=template_argument_help,
+)
 create_lua_pipeline.add_argument(
     "script", help="path of the Lua script", type=argparse.FileType("r")
 )
@@ -440,6 +454,7 @@ create_template_pipeline = subcommand(
             args.template,
             ocr_id=args.ocr,
             chat_id=args.chat,
+            use_vision=args.use_vision,
         )
     ),
 )
@@ -448,11 +463,16 @@ create_template_pipeline.add_argument(
 )
 create_template_pipeline.add_argument(
     "template",
-    type=template_argument,
+    type=json_data,
     help=template_argument_help,
 )
 create_template_pipeline.add_argument("--ocr", help="ID or alias of OCR resource")
 create_template_pipeline.add_argument("--chat", help="ID or alias of extraction LLM")
+create_template_pipeline.add_argument(
+    "--use-vision",
+    action=argparse.BooleanOptionalAction,
+    help="enable or disable LLM vision",
+)
 
 
 modify_pipeline = subcommand(
@@ -475,7 +495,7 @@ Any details not provided as a switch are left unchanged.
     ),
 )
 modify_pipeline.add_argument(
-    "pipeline", help="ID or alias of the pipeline to be changed."
+    "pipeline", help="ID or alias of the pipeline to be changed"
 )
 modify_pipeline.add_argument("--name", help="a new name for the pipeline")
 modify_pipeline.add_argument(
@@ -483,7 +503,7 @@ modify_pipeline.add_argument(
 )
 modify_pipeline.add_argument(
     "--template",
-    type=template_argument,
+    type=json_data,
     help=template_argument_help,
 )
 modify_pipeline.add_argument("--ocr", help="ID or alias of OCR resource")
@@ -494,7 +514,7 @@ modify_pipeline.add_argument(
 modify_pipeline.add_argument(
     "--use-vision",
     action=argparse.BooleanOptionalAction,
-    help="Enable or disable LLM vision.",
+    help="enable or disable LLM vision",
 )
 
 
@@ -533,7 +553,7 @@ run_anonymous_pipeline.add_argument(
 run_anonymous_pipeline.add_argument(
     "-t",
     "--template",
-    type=template_argument,
+    type=json_data,
     help=template_argument_help,
 )
 
@@ -574,7 +594,7 @@ create_inbox = subcommand(
         get_client(args).create_inbox(args.name, args.pipeline, ocr_id=args.ocr)
     ),
 )
-create_inbox.add_argument("name", help="Name of the inbox.")
+create_inbox.add_argument("name", help="name of the inbox")
 create_inbox.add_argument("pipeline", help="ID or alias of the extraction pipeline")
 create_inbox.add_argument("--ocr", help="OCR used for document display in the web UI")
 
@@ -596,9 +616,9 @@ recomputed.
     ),
 )
 modify_inbox.add_argument("inbox", help="ID of the inbox")
-modify_inbox.add_argument("--name", help="New name of the inbox")
+modify_inbox.add_argument("--name", help="new name of the inbox")
 modify_inbox.add_argument("--pipeline", help="ID of the extraction pipeline")
-modify_inbox.add_argument("--ocr", help="OCR used in document display in frontend.")
+modify_inbox.add_argument("--ocr", help="OCR used in document display in frontend")
 
 
 create_document = subcommand(
@@ -620,12 +640,10 @@ create_document.add_argument(
 list_documents = subcommand(
     "list-documents",
     group="Inboxes",
-    description="List documents in the inbox.",
+    description="List documents in the given inbox.",
     handler=lambda args: get_dumper(args)(get_client(args).list_documents(args.inbox)),
 )
-list_documents.add_argument(
-    "inbox", help="Specify UUID of the inbox containing the documents."
-)
+list_documents.add_argument("inbox", help="ID or alist of the inbox")
 
 
 list_extractions = subcommand(
@@ -645,7 +663,7 @@ list_inbox_jobs = subcommand(
     description="List pipeline runs triggered by documents of the inbox.",
     handler=lambda args: get_dumper(args)(get_client(args).list_inbox_jobs(args.inbox)),
 )
-list_inbox_jobs.add_argument("inbox", help="ID of the inbox.")
+list_inbox_jobs.add_argument("inbox", help="ID of the inbox")
 
 ### Documents
 
@@ -702,7 +720,7 @@ set_document_extraction = subcommand(
 set_document_extraction.add_argument("document", help="ID of the document")
 set_document_extraction.add_argument(
     "extraction",
-    type=json_argument,
+    type=json_data,
     help=f"new extraction data, {json_argument_help}",
 )
 
@@ -746,16 +764,23 @@ def do_request(args: argparse.Namespace) -> None:
         )
     method = args.method or ("POST" if (args.file or args.json) else "GET")
     params = dict(args.param) if args.param else None
+    headers = dict(args.header) if args.header else None
     files = dict(args.file) if args.file else None
     r = client._request(
         method,
         args.endpoint,
+        headers=headers,
         params=params,
         files=files,
         json=args.json,
     )
-    if r.headers.get("content-type") == "application/json":
+    content_type = r.headers.get("content-type")
+    if content_type == "application/json":
         dump(r.json())
+    elif args.output_file.isatty():
+        raise SystemExit(f"Use --output-file to save {content_type} content to a file")
+    else:
+        args.output_file.buffer.write(r.content)
 
 
 request = subcommand(
@@ -777,6 +802,14 @@ request.add_argument(
     help="request method, such as GET, POST, PUT, PATCH, DELETE"
     " (default: GET, or POST if a request body is included)",
 )
+request.add_argument(
+    "-H",
+    "--header",
+    help="extra headers to include in the request",
+    action="append",
+    metavar="KEY=VALUE",
+    type=key_value_argument,
+)
 request.add_argument(
     "-p",
     "--param",
@@ -799,7 +832,7 @@ request.add_argument(
     "-j",
     "--json",
     help=f"request body data, {json_argument_help}",
-    type=json_argument,
+    type=json_data,
 )
 
 
